Package: qqconf
Type: Package
Title: Creates Simultaneous Testing Bands for QQ-Plots
Version: 1.3.2
Authors@R: c(
    person("Eric", "Weine", email = "ericweine15@gmail.com", role = c("aut", "cre")),
    person("Mary Sara", "McPeek", email = "mcpeek@uchicago.edu", role = "aut"),
    person("Abney", "Mark", email = "abney@uchicago.edu", role = "aut"))
Description: Provides functionality for creating Quantile-Quantile (QQ) and Probability-Probability (PP) plots with simultaneous 
    testing bands to asses significance of sample deviation from a reference distribution <doi:10.18637/jss.v106.i10>.
License: GPL-3
Depends: R (>= 4.0.0)
SystemRequirements: fftw3 (>= 3.1.2)
Encoding: UTF-8
RoxygenNote: 7.2.3
Imports: MASS (>= 7.3-50), robustbase (>= 0.93-4), Rcpp
Suggests: knitr, rmarkdown, distr (>= 2.8.0)
Collate: 'one_sided.R' 'ppplot.R' 'qqconf-package.R' 'qqplot.R'
        'RcppExports.R' 'two_sided.R' 'utils.R'
VignetteBuilder: knitr
LinkingTo: Rcpp
URL: https://github.com/eweine/qqconf
BugReports: https://github.com/eweine/qqconf/issues
NeedsCompilation: yes
Packaged: 2023-04-14 19:38:46 UTC; ericweine
Author: Eric Weine [aut, cre],
  Mary Sara McPeek [aut],
  Abney Mark [aut]
Maintainer: Eric Weine <ericweine15@gmail.com>
Repository: CRAN
Date/Publication: 2023-04-14 23:00:21 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 06:43:20 UTC; unix
Archs: qqconf.so.dSYM

Package: qqconf
Type: Package
Title: Creates Simultaneous Testing Bands for QQ-Plots
Version: 1.3.2
Authors@R: c(
    person("Eric", "Weine", email = "ericweine15@gmail.com", role = c("aut", "cre")),
    person("Mary Sara", "McPeek", email = "mcpeek@uchicago.edu", role = "aut"),
    person("Abney", "Mark", email = "abney@uchicago.edu", role = "aut"))
Description: Provides functionality for creating Quantile-Quantile (QQ) and Probability-Probability (PP) plots with simultaneous 
    testing bands to asses significance of sample deviation from a reference distribution <doi:10.18637/jss.v106.i10>.
License: GPL-3
Depends: R (>= 4.0.0)
SystemRequirements: fftw3 (>= 3.1.2)
Encoding: UTF-8
RoxygenNote: 7.2.3
Imports: MASS (>= 7.3-50), robustbase (>= 0.93-4), Rcpp
Suggests: knitr, rmarkdown, distr (>= 2.8.0)
Collate: 'one_sided.R' 'ppplot.R' 'qqconf-package.R' 'qqplot.R'
        'RcppExports.R' 'two_sided.R' 'utils.R'
VignetteBuilder: knitr
LinkingTo: Rcpp
URL: https://github.com/eweine/qqconf
BugReports: https://github.com/eweine/qqconf/issues
NeedsCompilation: yes
Packaged: 2023-04-14 19:38:46 UTC; ericweine
Author: Eric Weine [aut, cre],
  Mary Sara McPeek [aut],
  Abney Mark [aut]
Maintainer: Eric Weine <ericweine15@gmail.com>
Repository: CRAN
Date/Publication: 2023-04-14 23:00:21 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:41:44 UTC; unix
Archs: qqconf.so.dSYM

# Generated by roxygen2: do not edit by hand

export(get_bounds_one_sided)
export(get_bounds_two_sided)
export(get_level_from_bounds_one_sided)
export(get_level_from_bounds_two_sided)
export(get_qq_band)
export(pp_conf_plot)
export(qq_conf_plot)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,abline)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(stats,median)
importFrom(stats,pnorm)
importFrom(stats,ppoints)
importFrom(stats,qbeta)
importFrom(stats,qnorm)
importFrom(stats,sd)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(qqconf)
useDynLib(qqconf, .registration = TRUE)

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----eval = F-----------------------------------------------------------------
#  install.packages("qqconf")

## ----message = F--------------------------------------------------------------
require(qqconf)

## -----------------------------------------------------------------------------
set.seed(0)
sample <- rnorm(n = 100, mean = 0, sd = 1)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1)
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4)
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  xlab = "More Informative Title"
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  points_params = list(cex = .5) # makes points smaller
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  line_params = list(col="red") # makes expectation line red
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  line_params = list(type="n") # removes expectation line
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  polygon_params = list(col = 'powderblue', border = NA) # change shading and keep no border
)

## -----------------------------------------------------------------------------
sample <- rbeta(n = 10000, shape1 = 1.0, shape2 = 1.05)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1)
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  difference = TRUE, # Make y-axis differenced
  ylim = c(-.0225, .0225)
)

## -----------------------------------------------------------------------------
mix <- distr::UnivarMixingDistribution(
  distr::Beta(shape1 = .25, shape2 = 1), 
  distr::Unif(),
  mixCoeff=c(.01, .99)
)

sampler <- distr::r(mix)

sample <- sampler(10000)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1)
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  difference = TRUE
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  log10 = TRUE
)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
bounds <- get_bounds_two_sided(alpha = .05, n = 100)

## ----fig.align = "center", fig.width = 7, fig.height = 5----------------------
bounds <- get_bounds_one_sided(alpha = .05, n = 100)

## ----session-info-------------------------------------------------------------
sessionInfo()

#' QQ Plot with Simultaneous and Pointwise Testing Bounds.
#'
#' Create a qq-plot with with a shaded simultaneous acceptance region and,
#' optionally, lines for a point-wise region. The observed values are plotted
#' against their expected values had they come from the specified distribution.
#'
#' If any of the points of the qq-plot fall outside the simultaneous acceptance region for the selected
#' level alpha test, that means that we can reject the null hypothesis that the data are i.i.d. draws from the
#' specified distribution. If \code{difference} is set to TRUE, the vertical axis plots the
#' observed quantile minus expected quantile. If pointwise bounds are used, then on average, alpha * n of the points will fall outside
#' the bounds under the null hypothesis, so the chance that the qq-plot has any points falling outside of the pointwise bounds
#' is typically much higher than alpha under the null hypothesis. For this reason, a simultaneous region is preferred.
#'
#' @param obs The observed data.
#' @param distribution The quantile function for the specified distribution. Defaults to \code{qnorm}.
#' Custom distributions are allowed as long as all parameters are supplied in dparams.
#' @param method Method for simultaneous testing bands. Must be either "ell" (equal local levels test), which applies a level \eqn{\eta} pointwise
#' test to each order statistic such that the Type I error of the global test is \code{alpha}, or "ks" to apply a
#' Kolmogorov-Smirnov test. "ell" is recommended.
#' @param alpha Type I error of global test of whether the data come from the reference distribution.
#' @param difference Whether to plot the difference between the observed and
#'   expected values on the vertical axis.
#' @param log10 Whether to plot axes on -log10 scale (e.g. to see small p-values). Can only be used for strictly
#' positive distributions.
#' @param right_tail This argument is only used if \code{log10} is \code{TRUE}. When \code{TRUE},
#' the x-axis is -log10(1 - Expected Quantile) and the y-axis is -log10(1 - Observed Quantile).
#' When \code{FALSE} (default) the x-axis is -log10(Expected Quantile) and the y-axis is
#' -log10(Observed Quantile). The argument should be set to \code{TRUE} only when the support
#' of the distribution lies in (0, 1), and one wants to make
#' observations in the right tail of the distribution easier to see. The argument should be
#' set to \code{FALSE} when one wants to make
#' observations in the left tail of the distribution easier to see.
#' @param add Whether to add points to an existing plot.
#' @param dparams List of additional arguments for the quantile function of the distribution
#'   (e.g. df=1). Note that if any parameters of the distribution are specified, parameter estimation will not be performed
#'   on the unspecified parameters, and instead they will take on the default values set by the distribution function.
#'   For the uniform distribution, parameter estimation is not performed, and
#'   the default parameters are max = 1 and min = 0.
#'   For other distributions parameters will be estimated if not provided.
#'   For the normal distribution, we estimate the mean as the median and the standard deviation as \eqn{Sn} from the paper by Rousseeuw and Croux 1993
#'   "Alternatives to the Median Absolute Deviation". For all other distributions besides uniform and normal,
#'   the code uses MLE to estimate the parameters. Note that estimation is not implemented for custom distributions, so all
#'   parameters of the distribution must be provided by the user.
#' @param bounds_params List of optional arguments for \code{get_bounds_two_sided}
#'   (i.e. \code{tol}, \code{max_it}, \code{method}).
#' @param line_params Arguments passed to the \code{lines} function to modify the line that indicates a perfect fit of the
#'   reference distribution.
#' @param plot_pointwise Boolean indicating whether pointwise bounds should be added to the plot
#' @param pointwise_lines_params Arguments passed to the \code{lines} function that modifies pointwise bounds when plot_pointwise is
#'   set to TRUE.
#' @param points_params Arguments to be passed to the \code{points} function to plot the data.
#' @param polygon_params Arguments to be passed to the polygon function to construct simultaneous confidence region.
#'   By default \code{border} is set to NA and \code{col} is set to grey.
#' @param prob_pts_method (optional) method used to get probability points for plotting.
#' The quantile function will be applied to these points to
#' get the expected values.  When this argument is set to \code{"normal"}
#' (recommended for a normal QQ plot) \code{ppoints(n)} will be used,  which is what
#' most other plotting software uses. When this argument is set to \code{"uniform"}
#' (recommended for a uniform QQ plot) \code{ppoints(n, a=0)}, which are the expected
#' values of the order statistics of Uniform(0, 1), will be used.  Finally,
#'  when this argument is set to \code{"median"} (recommended for all other
#'  distributions) \code{qbeta(.5, c(1:n), c(n:1))} will be used. Under the default
#'  setting, \code{"best_available"}, the probability points as recommended above will
#'  be used. Note that \code{"median"} is suitable for all distributions and is
#'  particularly recommended when alpha is large.
#' @param ... Additional arguments passed to the plot function.
#'
#' @importFrom stats median pnorm ppoints qbeta qnorm
#' @importFrom graphics abline lines plot points polygon
#'
#' @export
#'
#' @return None, QQ plot is produced.
#'
#' @examples
#' set.seed(0)
#' smp <- runif(100)
#'
#' # Plot QQ plot against uniform(0, 1) distribution
#' qq_conf_plot(
#'   obs=smp,
#'   distribution = qunif
#' )
#'
#' # Make same plot on -log10 scale to highlight small p-values,
#' # with radius of plot circles also reduced by .5
#' qq_conf_plot(
#'   obs=smp,
#'   distribution = qunif,
#'   points_params = list(cex = .5),
#'   log10 = TRUE
#' )
#'
#' # Make same plot with difference between observed and expected values on the y-axis
#' qq_conf_plot(
#'   obs=smp,
#'   distribution = qunif,
#'   difference = TRUE
#' )
#'
#' # Make same plot with sample plotted as a blue line, expected value line plotted as a red line,
#' # and with pointwise bounds plotted as black lines
#' qq_conf_plot(
#'   obs=smp,
#'   distribution = qunif,
#'   plot_pointwise = TRUE,
#'   points_params = list(col="blue", type="l"),
#'   line_params = list(col="red")
#' )
#'

qq_conf_plot <- function(obs,
                         distribution = qnorm,
                         method = c("ell", "ks"),
                         alpha = 0.05,
                         difference = FALSE,
                         log10 = FALSE,
                         right_tail = FALSE,
                         add = FALSE,
                         dparams = list(),
                         bounds_params = list(),
                         line_params = list(),
                         plot_pointwise = FALSE,
                         pointwise_lines_params = list(),
                         points_params = list(),
                         polygon_params = list(border = NA, col = 'gray'),
                         prob_pts_method = c("best_available", "normal", "uniform", "median"),
                         ...) {

  if (!("p" %in% names(formals(distribution)))) {

    stop("distribution function must take 'p' as an argument.
         Did you mean to make a PP plot?")

  }

  dots <- list(...)
  method <- match.arg(method)
  if ( is.null(dots$ylab)) {
    if (difference && log10 && right_tail) {
      ylab <- expression("-log"[10]*"(1 - Observed quantiles) + log"[10]*"(1 - Expected quantiles)")
    } else if(difference && log10) {
      ylab <- expression("-log"[10]*"(Observed quantiles) + log"[10]*"(Expected quantiles)")
    } else if(difference) {
      ylab <- 'Obseved quantiles - Expected quantiles'
    } else if(log10 && right_tail) {
      ylab <- expression("-log"[10]*"(1 - Observed quantiles)")
    }else if (log10) {
      ylab <- expression("-log"[10]*"(Observed quantiles)")
    } else {
      ylab <- 'Observed quantiles'
    }
  } else {
    ylab <- dots$ylab
    dots <- within(dots, rm(ylab))
  }
  if (is.null(dots$xlab)) {
    if (log10 && right_tail) {
      xlab <- expression("-log"[10]*"(1 - Expected quantiles)")
    } else if(log10) {
      xlab <- expression("-log"[10]*"(Expected quantiles)")
    }
    else {
      xlab <- 'Expected quantiles'
    }
  } else {
    xlab <- dots$xlab
    dots <- within(dots, rm(xlab))
  }

  samp_size <- length(obs)
  conf_int <- 1 - alpha
  conf <- c(alpha / 2, conf_int + alpha / 2)
  ## The observed and expected quantiles. Expected quantiles are based on the specified
  ## distribution
  # constant for visual expansion of confidence regions
  c <- .5
  obs_pts <- sort(obs)

  dist_name <- as.character(substitute(distribution))
  if(length(dparams) == 0 && dist_name == "qunif") {

    dparams['min'] <- 0
    dparams['max'] <- 1

  } else if (length(dparams) == 0) {

    cat("no dparams supplied. Estimating parameters from the data...\n")
    MASS_name <- get_mass_name_from_distr(dist_name, "qq")
    dparams <- estimate_params_from_data(MASS_name, obs)

  }

  prob_pts_method <- match.arg(prob_pts_method)

  if (prob_pts_method == "best_available") {

    prob_pts_method <- get_best_available_prob_pts_method(dist_name)

  }

  global_bounds <- get_qq_band(
    obs = obs,
    alpha = alpha,
    distribution = distribution,
    dparams = dparams,
    ell_params = bounds_params,
    band_method = method,
    prob_pts_method = prob_pts_method
  )

  global_low <- global_bounds$lower_bound
  global_high <- global_bounds$upper_bound

  exp_pts <- global_bounds$expected_value

  ext_quantile <- get_extended_quantile(prob_pts_method, samp_size)

  if (log10 && right_tail) {

    exp_pts <- -log10(1 - exp_pts)
    low_exp_pt <- c * -log10(do.call(distribution, c(list(p=ext_quantile$high_pt), dparams))) + (1 - c) * exp_pts[1]
    high_exp_pt <- c * -log10(do.call(distribution, c(list(p=ext_quantile$low_pt), dparams))) + (1 - c) * exp_pts[samp_size]
    if (any(obs_pts <= 0)) {

      stop("log10 scaling can only be used with strictly positive distributions. Consider using pp_conf_plot.")

    }
    obs_pts <- -log10(1 - obs_pts)

  }
  else if (log10 == TRUE) {

    exp_pts <- -log10(exp_pts)
    low_exp_pt <- c * -log10(do.call(distribution, c(list(p=ext_quantile$low_pt), dparams))) + (1 - c) * exp_pts[1]
    high_exp_pt <- c * -log10(do.call(distribution, c(list(p=ext_quantile$high_pt), dparams))) + (1 - c) * exp_pts[samp_size]
    if (any(obs_pts <= 0)) {

      stop("log10 scaling can only be used with strictly positive distributions. Consider using pp_conf_plot.")

    }
    obs_pts <- -log10(obs_pts)

  }
  else {

    low_exp_pt <- c * do.call(distribution, c(list(p=ext_quantile$low_pt), dparams)) + (1 - c) * exp_pts[1]
    high_exp_pt <- c * do.call(distribution, c(list(p=ext_quantile$high_pt), dparams)) + (1 - c) * exp_pts[samp_size]

  }
  if (difference) {
    y_pts <- obs_pts - exp_pts
  } else {
    y_pts <- obs_pts
  }

  ## When not adding points to a qq-plot compute pointwise and global confidence bounds.
  if (!add) {
    left <- exp_pts[1]
    right <- exp_pts[samp_size]
    bottom <- min(y_pts) #obs_pts[1]
    top <- max(y_pts) #obs_pts[samp_size]
    do.call(plot, c(list(x=c(left, right), y=c(bottom, top), type='n', xlab=xlab, ylab=ylab), dots))
    pointwise_low <- do.call(distribution,
                             c(list(p=qbeta(conf[1], 1:samp_size, samp_size:1)), dparams))
    pointwise_high <- do.call(distribution,
                              c(list(p=qbeta(conf[2], 1:samp_size, samp_size:1)), dparams))

    if(log10 == TRUE && right_tail == TRUE) {

      pointwise_low <- -log10(1 - pointwise_low)
      pointwise_high <- -log10(1 - pointwise_high)
      global_low <- -log10(1 - global_low)
      global_high <- -log10(1 - global_high)

    }
    else if (log10 == TRUE) {

      pointwise_low <- -log10(pointwise_low)
      pointwise_high <- -log10(pointwise_high)
      global_low <- -log10(global_low)
      global_high <- -log10(global_high)

    }

    if ("ylim" %in% names(dots)) {

      bottom <- dots$ylim[1] - 1000
      top <- dots$ylim[2] + 1000

    } else {

      global_low_temp <- global_low[is.finite(global_low)]
      global_high_temp <- global_high[is.finite(global_high)]
      bottom <- min(global_low_temp) - 1000
      top <- max(global_high_temp) + 1000

    }

    if (difference) {

      low_global_diff <- global_low - exp_pts
      low_global_diff <- c(low_global_diff[1], low_global_diff, low_global_diff[samp_size])
      high_global_diff <- global_high - exp_pts
      high_global_diff <- c(high_global_diff[1], high_global_diff, high_global_diff[samp_size])
      low_pointwise_diff <- pointwise_low - exp_pts
      low_pointwise_diff <- c(low_pointwise_diff[1], low_pointwise_diff, low_pointwise_diff[samp_size])
      high_pointwise_diff <- pointwise_high - exp_pts
      high_pointwise_diff <- c(high_pointwise_diff[1], high_pointwise_diff, high_pointwise_diff[samp_size])
      exp_pts <- c(low_exp_pt, exp_pts, high_exp_pt)

      do.call(
        polygon,
        c(list(x = c(exp_pts, rev(exp_pts)),
               y = pmin(pmax(c(low_global_diff, rev(high_global_diff)), bottom), top)),
          polygon_params)
      )
      if (plot_pointwise) {

        do.call(lines, c(list(x = exp_pts, y = low_pointwise_diff), pointwise_lines_params))
        do.call(lines, c(list(x = exp_pts, y = high_pointwise_diff), pointwise_lines_params))

      }

    } else {

      # code to extend region for visibility
      global_low <- c(global_low[1], global_low, global_low[samp_size])
      global_high <- c(global_high[1], global_high, global_high[samp_size])
      pointwise_low <- c(pointwise_low[1], pointwise_low, pointwise_low[samp_size])
      pointwise_high <- c(pointwise_high[1], pointwise_high, pointwise_high[samp_size])
      exp_pts <- c(low_exp_pt, exp_pts, high_exp_pt)

      do.call(
        polygon,
        c(list(x = c(exp_pts, rev(exp_pts)),
               y = pmin(pmax(c(global_low, rev(global_high)), bottom), top)),
          polygon_params)
        )
      if (plot_pointwise) {

        do.call(lines, c(list(x = exp_pts, y = pointwise_low), pointwise_lines_params))
        do.call(lines, c(list(x = exp_pts, y = pointwise_high), pointwise_lines_params))

      }

    }

    do.call(points, c(list(x = exp_pts[2:(samp_size + 1)], y = y_pts), points_params))

  } else {

    do.call(points, c(list(x = exp_pts, y = y_pts), points_params))

  }

  if (difference) {

    do.call(lines, c(list(x = c(min(exp_pts), max(exp_pts)), y = c(0, 0)), line_params))

  } else{

    do.call(lines, c(list(x = c(min(exp_pts), max(exp_pts)), y = c(min(exp_pts), max(exp_pts))), line_params))

  }

}

#' Shorthand for two numerical comparisons
#'
#' @param x numeric value
#' @param gte lower bound
#' @param lte upper bound
#'
#' @return boolean
between <- function(x, gte, lte) {

  x >= gte & x <= lte

}

#' Convert R Distribution Function to MASS Distribution Name
#'
#' @param distr R distribution function (e.g. qnorm or pnorm)
#' @param band_type one of "qq" (for quantile functions) or "pp" (
#' for probability functions).
#'
#' @return string of MASS distribution name
get_mass_name_from_distr <- function(distr, band_type) {

  if (band_type == "qq") {

    corresp_dn <- function(distributionName) {
      switch(
        distributionName,
        qbeta = "beta",
        qcauchy = "cauchy",
        qchisq = "chi-squared",
        qexp = "exponential",
        qf = "f",
        qgamma = "gamma",
        qgeom = "geometric",
        qnorm = "normal",
        qlnorm = "log-normal",
        qlogis = "logistic",
        qnorm = "normal",
        qnbinom = "negative binomial",
        qpois = "poisson",
        qt = "t",
        qweibull = "weibull",
        NULL
      )
    }

  } else if (band_type == "pp") {

    corresp_dn <- function(distributionName) {
      switch(
        distributionName,
        pbeta = "beta",
        pcauchy = "cauchy",
        pchisq = "chi-squared",
        pexp = "exponential",
        pf = "f",
        pgamma = "gamma",
        pgeom = "geometric",
        pnorm = "normal",
        plnorm = "log-normal",
        plogis = "logistic",
        pnorm = "normal",
        pnbinom = "negative binomial",
        ppois = "poisson",
        pt = "t",
        pweibull = "weibull",
        NULL
      )
    }

  }

  dist <- corresp_dn(distr)
  if(is.null(dist)) {

    stop("Unknown distribution provided")

  }
  return(dist)

}

#' Estimate Parameters from Data
#'
#' For select distributions, parameters are estimated from data. Generally,
#' the MLEs are used. However, for the normal distribution we use robust
#' estimators.
#'
#' @param distr_name \code{MASS} name of distribution
#' @param obs observation vector
#'
#' @importFrom stats sd
#'
#' @return list of distribution parameters
estimate_params_from_data <- function(distr_name, obs) {

  # initial value for some distributions
  initVal <- function(distributionName) {
    switch(
      distributionName,
      "beta" = list(shape1 = 1, shape2 = 1),
      "chi-squared" = list(df = 1),
      "f" = list(df1 = 1, df2 = 2),
      NULL
    )
  }

    if(is.null(initVal(distr_name))) {
      if(distr_name == "normal") {

        # use special estimators for the normal distribution
        dparams <- list()
        dparams['mean'] <- median(x = obs)
        dparams['sd'] <- robustbase::Sn(x = obs)

      } else if (distr_name == "t") {

        dparams <- list()
        suppressWarnings({
          param_ests <- MASS::fitdistr(
            x = obs,
            densfun = "t",
            start = list(df = 2, m = 1),
            s = 1
          )$estimate
        })

        dparams['ncp'] <- param_ests['m']
        dparams['df'] <- param_ests['df']

      }
      else {

        suppressWarnings({
          dparams <- MASS::fitdistr(x = obs, densfun = distr_name)$estimate
        })

      }

    } else {

      suppressWarnings({
        dparams <- MASS::fitdistr(
          x = obs,
          densfun = distr_name,
          start = initVal(distr_name))$estimate
      })

    }


  return(dparams)

}

#' Get Best Available Method for Probability Points
#'
#' Determines name of best method for obtaining expected points for a qq or pp
#' plot.
#'
#' @param dist_name character name of distribution
#'
#' @return character name of best expected points method
#'
get_best_available_prob_pts_method <- function(dist_name) {

  if (dist_name %in% c("qunif", "punif")) {

    return("uniform")

  } else if (dist_name %in% c("qnorm", "pnorm")) {

    return("normal")

  } else {

    return("median")

  }

}

#' Get Quantile for First and Last Point of QQ or PP Plot
#'
#' @param exp_pts_method method used to derive expected points
#' @param n sample size
#'
#' @return list with low and high point
#'
get_extended_quantile <- function(exp_pts_method, n) {

  if (exp_pts_method == "uniform") {

    high_pt <- 1 - (1 / max(n * 1.25, n + 2))
    low_pt <- 1 / max(n * 1.25, n + 2)

  } else if (exp_pts_method == "median"){

    new_samp_size <- floor(n * 1.02)
    ppoints_adj <- ppoints(new_samp_size)
    low_pt <- ppoints_adj[1]
    high_pt <- ppoints_adj[new_samp_size]

  } else if (exp_pts_method == "normal") {

    new_samp_size <- floor(n * 1.3)
    ppoints_adj <- ppoints(new_samp_size)
    low_pt <- ppoints_adj[1]
    high_pt <- ppoints_adj[new_samp_size]

  }

  return(
    list(
      low_pt = low_pt,
      high_pt = high_pt
    )
  )

}

#' Get Quantile Distribution from Probability Distribution
#'
#' @param dname probability distribution (e.g. \code{pnorm})
#'
#' @return quantile distribution (e.g. \code{qnorm}).
#'
get_qq_distribution_from_pp_distribution <- function(dname) {

  dname_no_p <- substr(dname, 2, nchar(dname))
  eval(parse(text = paste0("q", dname_no_p)))

}

#' Check Validity of Two-Sided Bounds
#'
#' Given bounds for a two sided test, this checks that none of
#' the bounds fall outside of [0, 1] and that all upper bounds
#' are greater than the corresponding lower bounds.
#' This also ensures the the length of the bounds are the same.
#' This not meant to be called by the user.
#'
#' @param lower_bounds Numeric vector where the ith component is the lower bound
#' for the ith order statistic.
#' @param upper_bounds Numeric vector where the ith component is the lower bound
#' for the ith order statistic.
#'
#' @return None
#'
check_bounds_two_sided <- function(lower_bounds,
                                   upper_bounds) {

  if(any(lower_bounds > 1) || any(lower_bounds < 0)) {

    stop("Not all lower bounds between 0 and 1 (inclusive)")

  }

  if(any(upper_bounds > 1) || any(upper_bounds < 0)) {

    stop("Not all upper bounds between 0 and 1 (inclusive)")

  }

  if(any(upper_bounds - lower_bounds < 0)) {

    stop("Not all upper bounds are greater than their corresponding lower bounds")

  }

  if(length(lower_bounds) != length(upper_bounds)) {

    stop("The length of the bounds differ")

  }

  if(is.unsorted(lower_bounds)) {

    stop("Only lower bounds in ascending order are supported")

  }

  if(is.unsorted(upper_bounds)) {

    stop("Only upper bounds in ascending order are supported")

  }

}


#' Monte Carlo Simulation for Two-Sided Test
#'
#' Given bounds for a two sided test on uniform order statistics, this computes
#' the Type I Error Rate \eqn{\alpha} using simulations.
#'
#' @param lower_bounds Numeric vector where the ith component is the lower bound
#' for the ith order statistic. The components must be distinct values in (0, 1) that
#' are in ascending order.
#' @param upper_bounds Numeric vector where the ith component is the lower bound
#' for the ith order statistic. The values must be in ascending order and
#' the ith component must be larger than the ith component of the lower bounds.
#' @param num_sims (optional) Number of simulations to be run, 1 Million by default.
#'
#' @return Type I Error Rate \eqn{\alpha}
#'
monte_carlo_two_sided <- function(lower_bounds,
                                  upper_bounds,
                                  num_sims = 1000000) {

  check_bounds_two_sided(lower_bounds, upper_bounds)
  n <- length(lower_bounds)

  num_in_ci <- 0 # tracks how many sims have all points in the confidence intervals

  for(iter in seq(from = 1, to = num_sims, by = 1)) {

    # generate uniform RVs
    iter_rvs <- stats::runif(n)
    # sort them
    iter_order_stats <- sort(iter_rvs)

    if (all(iter_order_stats > lower_bounds) && all(iter_order_stats < upper_bounds)) {

      num_in_ci <- num_in_ci + 1

    }

  }

  alpha <- 1 - (num_in_ci / num_sims)
  return(alpha)

}

#' Calculates Approximate Local Level
#'
#' This function uses the approximation from Gontscharuk & Finner's Asymptotics of
#' goodness-of-fit tests based on minimum p-value statistics (2017) to approximate
#' local levels for finite sample size. We use these authors constants for \eqn{\alpha} = .1, and .05,
#' and for \eqn{\alpha} = .01 we use a slightly different approximation.
#'
#' @param n Number of tests to do
#' @param alpha Global type I error rate \eqn{\alpha} of the tests
#'
#' @return Approximate local level
get_asymptotic_approx_corrected_alpha <- function(n, alpha) {

  alpha_epsilon <- 10 ^ (-5)
  if (between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon)) {

    c_alpha <- 1.591

  } else if (between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon)) {

    c_alpha <- 1.3

  } else if (between(alpha, .1 - alpha_epsilon, .1 + alpha_epsilon)) {

    c_alpha <- 1.1

  }

  eta_approx = -log(1-alpha)/(2*log(log(n))*log(n))*(1-c_alpha*log(log(log(n)))/log(log(n)))
  return(eta_approx)

}

#' Calculates Global Significance Level From Simultaneous Two-Sided Bounds for Rejection Region
#'
#' For a test of uniformity of i.i.d. observations on the unit interval, this function will determine the significance
#' level as a function of the rejection region. Suppose \eqn{n} observations are drawn i.i.d. from some CDF F(x) on the unit interval,
#' and it is desired to test the null hypothesis that F(x) = x for all x in (0, 1) against a two-sided alternative.
#' Suppose the acceptance region for the test is described by a set of intervals, one for each order statistic.
#' Given the bounds for these intervals, this function calculates the significance level of the test where the
#' null hypothesis is rejected if at least one of the order statistics is outside its corresponding interval.
#'
#' Uses the method of Moscovich and Nadler (2016) as implemented in Crossprob (Moscovich 2020).
#'
#' @param lower_bounds Numeric vector where the ith component is the lower bound for the acceptance interval
#' for the ith order statistic. The components must lie in [0, 1], and each component must be greater than
#' or equal to the previous one.
#' @param upper_bounds Numeric vector of the same length as \code{lower_bounds} where the ith component is the upper bound
#' for the acceptance interval for the ith order statistic. The components must lie in [0, 1], and each component must be
#' greater than or equal to the previous one. In addition,
#' the ith component of \code{upper_bounds} must be greater than or equal to the ith component of \code{lower_bounds}.
#'
#' @return Global Significance Level \eqn{\alpha}
#'
#' @examples
#' # For X1, X2 iid unif(0,1), calculate 1 - P(.1 < min(X1, X2) < .6 and .5 < max(X1, X2) < .9)
#' get_level_from_bounds_two_sided(lower_bounds = c(.1, .5), upper_bounds = c(.6, .9))
#'
#' # Finds the global significance level corresponding to the local level eta.
#' # Suppose we reject the null hypothesis that X1, ..., Xn are iid unif(0, 1) if and only if at least
#' # one of the order statistics X(i) is significantly different from
#' # its null distribution based on a level-eta
#' # two-sided test, i.e. we reject if and only if X(i) is outside the interval
#' # (qbeta(eta/2, i, n - i + 1), qbeta(1 - eta/2, i, n - i + 1)) for at least one i.
#' # The lines of code below calculate the global significance level of
#' # the test (which is necessarily larger than eta if n > 1).
#' n <- 100
#' eta <- .05
#' lb <- qbeta(eta / 2, c(1:n), c(n:1))
#' ub <- qbeta(1 - eta / 2, c(1:n), c(n:1))
#' get_level_from_bounds_two_sided(lower_bounds = lb, upper_bounds = ub)
#'
#' @references
#' \itemize{
#' \item{\href{https://www.sciencedirect.com/science/article/abs/pii/S0167715216302802}{
#' Moscovich, Amit, and Boaz Nadler. "Fast calculation of boundary crossing probabilities for Poisson processes."
#' Statistics & Probability Letters 123 (2017): 177-182.}}
#' \item{\href{https://github.com/mosco/crossing-probability}{
#' Amit Moscovich (2020). Fast calculation of p-values for one-sided
#' Kolmogorov-Smirnov type statistics. arXiv:2009.04954}}
#' }
#'
#' @useDynLib qqconf
#'
#' @export
get_level_from_bounds_two_sided <- function(lower_bounds,
                                            upper_bounds) {

  check_bounds_two_sided(lower_bounds, upper_bounds)
  alpha <- 1 - fft_get_level_from_bounds_two_sided(lower_bounds, upper_bounds)
  return(alpha)

}

#' Calculates Rejection Region of Two-Sided Equal Local Levels Test.
#'
#' The context is that n i.i.d. observations are assumed to be drawn
#' from some distribution on the unit interval with c.d.f. F(x), and it is
#' desired to test the null hypothesis that F(x) = x for all x in (0,1),
#' referred to as the "global null hypothesis," against a two-sided alternative.
#' An "equal local levels" test is used, in which each of the n order statistics is
#' tested for significant deviation from its null distribution by a 2-sided test
#' with significance level \eqn{\eta}.  The global null hypothesis is rejected if at
#' least one of the order statistic tests is rejected at level \eqn{\eta}, where \eqn{\eta} is
#' chosen so that the significance level of the global test is alpha.
#' Given the size of the dataset n and the desired global significance level alpha,
#' this function calculates the local level \eqn{\eta} and the acceptance/rejection regions for the test.
#' There are a set of n intervals, one for each order statistic.
#' If at least one order statistic falls outside the corresponding interval,
#' the global test is rejected.
#'
#'
#' @param alpha Desired global significance level of the test.
#' @param n Size of the dataset.
#' @param tol (optional) Relative tolerance of the \code{alpha} level of the
#' simultaneous test. Defaults to 1e-8. Used only if \code{method} is set to
#' "search" or if method is set to "best_available" and the best available
#' method is a search.
#' @param max_it (optional) Maximum number of iterations of Binary Search Algorithm
#' used to find the bounds. Defaults to 100 which should be much larger than necessary
#' for a reasonable tolerance. Used only if \code{method} is set to
#' "search" or if method is set to "best_available" and the best available
#' method is a search.
#' @param method (optional) Argument indicating if the calculation should be done using a highly
#' accurate approximation, "approximate", or if the calculations should be done using an exact
#' binary search calculation, "search". The default is "best_available" (recommended), which uses the exact search
#' when either (i) the approximation isn't available or (ii) the approximation is available but isn't highly accurate and the search method
#' isn't prohibitively slow (occurs for small to moderate \code{n} with \code{alpha} = .1).
#' Of note, the approximate method is only available for alpha values of .1, .05, and .01. In the case of alpha = .05 or .01, the
#' approximation is highly accurate for all values of \code{n} up to at least \code{10^6}.
#'
#' @return A list with components
#' \itemize{
#'   \item lower_bound - Numeric vector of length \code{n} containing the lower bounds
#'   for the acceptance regions of the test of each order statistic.
#'   \item upper_bound - Numeric vector of length \code{n} containing the upper bounds
#'   for the acceptance regions of the test of each order statistic.
#'   \item x - Numeric vector of length \code{n} containing the expectation of each order statistic.
#'   These are the x-coordinates for the bounds if used in a pp-plot.
#'   The value is \code{c(1:n) / (n + 1)}.
#'   \item local_level - Significance level \eqn{\eta} of the local test on each individual order statistic. It is equal for all order
#'   statistics and will be less than \code{alpha} for all \code{n} > 1.
#' }
#'
#' @examples
#' get_bounds_two_sided(alpha = .05, n = 100)
#'
#' @importFrom utils head tail
#'
#' @export
get_bounds_two_sided <- function(alpha,
                                n,
                                tol = 1e-8,
                                max_it = 100,
                                method=c("best_available", "approximate", "search")) {

  if (alpha >= 1 || alpha <= 0) {

    stop("alpha must be between 0 and 1 (exclusive).")

  }

  if (as.integer(n) != n) {

    stop("n must be an integer")

  }

  if (n < 1) {

    stop("n must be greater than 0")

  }

  if (n == 1) {

    return(list(lower_bound = c(alpha / 2),
                upper_bound = c(1 - alpha / 2),
                x = c(.5),
                local_level = alpha))

  }

  n_param <- n
  if (n >= 10) {

    # Approximation only available for n < 10
    method <- match.arg(method)

  } else {

    method <- "search"

  }

  # Value used for testing if alpha is approximately equal to a set of pre-set values
  alpha_epsilon <- 10 ^ (-5)

  # Approximations are only available for alpha = .05 or alpha = .01
  if (method == "search") {

    eta_high <- alpha
    eta_low <- alpha / n
    eta_curr <- eta_low + (eta_high - eta_low) / 2
    n_it <- 0

    while (n_it < max_it) {

      n_it <- n_it + 1
      h_vals <- stats::qbeta(eta_curr / 2, 1:n, n:1)
      g_vals <- stats::qbeta(1 - (eta_curr / 2), 1:n, n:1)
      test_alpha <- get_level_from_bounds_two_sided(h_vals, g_vals)

      if (abs(test_alpha - alpha) / alpha <= tol) break

      if (test_alpha > alpha) {

        eta_high <- eta_curr
        eta_curr <- eta_curr - (eta_curr - eta_low) / 2

      } else if (test_alpha < alpha) {

        eta_low <- eta_curr
        eta_curr <- eta_curr + (eta_high - eta_curr) / 2

      }

      eta <- eta_curr

    }

    if(n_it == max_it) {

      warning("Maximum number of iterations reached.")

    }

  }

  else if (method == "approximate") {

    if (!(between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon) ||
          between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon) ||
          between(alpha, .1 - alpha_epsilon, .1 + alpha_epsilon))) {

      stop("The approximate method is only configured for alpha = .1, .05, .01. Consider setting method='search'")

    }

    if (between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon)) {

      lookup_table <- alpha_01_df

    } else if (between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon)) {

      lookup_table <- alpha_05_df

    }

    if (
      between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon) ||
        between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon)
        ) {

      if (n %in% lookup_table$n) {

        eta_df <- lookup_table[lookup_table$n == n_param, ]
        eta <- eta_df$local_level[1]

      } else if (
        (between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon) && n > 10 ^ 5) ||
          (between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon) && n > 10 ^ 5)) {

        eta <- get_asymptotic_approx_corrected_alpha(n, alpha)

      }
      else {

        # Do linear interpolation
        larger_n_df <- lookup_table[lookup_table$n > n_param, ]
        larger_n_df <- head(larger_n_df, 1)
        larger_n <- larger_n_df$n[1]
        larger_n_eta <- larger_n_df$local_level[1]

        smaller_n_df <- lookup_table[lookup_table$n < n_param, ]
        smaller_n_df <- tail(smaller_n_df, 1)
        smaller_n <- smaller_n_df$n[1]
        smaller_n_eta <- smaller_n_df$local_level[1]

        # y = mx + b
        m <- (larger_n_eta - smaller_n_eta) / (larger_n - smaller_n)
        b <- smaller_n_eta - m * smaller_n
        eta <- m * n + b

      }

    } else {

      eta <- get_asymptotic_approx_corrected_alpha(n, alpha)

    }

    h_vals <- stats::qbeta(eta / 2, 1:n, n:1)
    g_vals <- stats::qbeta(1 - (eta / 2), 1:n, n:1)

  }

  else if (method == "best_available") {

    # Don't return approximation for alpha = .1 and n <= 500, it's not accurate
    if (between(alpha, .1 - alpha_epsilon, .1 + alpha_epsilon) && n <= 500 ||
        !(between(alpha, .01 - alpha_epsilon, .01 + alpha_epsilon) ||
          between(alpha, .05 - alpha_epsilon, .05 + alpha_epsilon) ||
          between(alpha, .1 - alpha_epsilon, .1 + alpha_epsilon))
        ) {

      return(
        get_bounds_two_sided(
          alpha = alpha,
          n = n,
          tol = tol,
          max_it = max_it,
          method="search"
        )
      )

    } else {

      return(
        get_bounds_two_sided(
          alpha = alpha,
          n = n,
          tol = tol,
          max_it = max_it,
          method="approximate"
        )
      )

    }

  }

  alpha_vec <- seq(from = 1, to = n, by = 1)
  beta_vec <- n - alpha_vec + 1
  order_stats_mean <- alpha_vec / (alpha_vec + beta_vec)

  return(list(lower_bound = h_vals,
              upper_bound = g_vals,
              x = order_stats_mean,
              local_level = eta))

}

#' Create QQ Plot Testing Band
#'
#' Flexible interface for creating a testing band for a Quantile-Quantile (QQ)
#' plot.
#'
#' @param n,obs either a number of observations (specified by setting \code{n}),
#' or a numeric vector of observations (specified by setting \code{obs}). One
#' argument must be specified. If all parameters of \code{distribution}
#' are known, then the testing band only depends on the number of observations \code{n}.
#' Thus, providing \code{n} is simpler when all parameters
#' of \code{distribution} are known and specified via \code{dparams}
#' (or when using all default parameter choices of \code{distribution} is desired).
#' If estimating parameters from the data is preferred, \code{obs} should be
#' specified and estimation will take place as described in the documentation for
#' argument \code{dparams}.
#' @param alpha (optional) desired significance level of the testing band. If \code{method}
#' is set to \code{"ell"} or \code{"ks"}, then this is the global significance
#' level of the testing band. If \code{method} is set to \code{"pointwise"}, then
#' the band is equivalent to simply conducting a level \code{alpha} test on each
#' order statistic individually. Pointwise bands will generally have much
#' larger global Type I error than \code{alpha}. Defaults to \code{.05}.
#' @param distribution The quantile function for the specified distribution.
#' Defaults to \code{qnorm}, which is appropriate for testing normality
#' of the observations in a QQ plot.
#' @param dparams (optional)  List of additional arguments for the \code{distribution} function
#'   (e.g. df=1). If \code{obs} is not specified and this argument is left blank,
#'   the default arguments of \code{distribution} are used. If \code{obs} is specified and this argument is left blank,
#'   parameters are estimated from the data (except if \code{distribution} is set to \code{qunif},
#'   in which case no estimation occurs and the default parameters are \code{max = 1} and \code{min = 0}).
#'   For the normal distribution, we estimate the mean as the median and the standard deviation as \eqn{Sn} from the paper by Rousseeuw and Croux 1993
#'   "Alternatives to the Median Absolute Deviation". For all other distributions besides uniform and normal,
#'   the code uses MLE to estimate the parameters. Note that if any parameters of the distribution are specified
#'   in \code{dparams}, parameter estimation will not be performed
#'   on the unspecified parameters, and instead they will take on the default values
#'   set by \code{distribution}.
#' @param ell_params (optional) list of optional arguments for \code{get_bounds_two_sided}
#'   (i.e. \code{tol}, \code{max_it}, \code{method}). Only used if \code{method}
#'   is set to \code{"ell"}
#' @param band_method (optional) method for creating the testing band. The default,
#' \code{"ell"} uses the equal local levels method
#' (see \code{get_bounds_two_sided} for more information). \code{"ks"} uses
#' the Kolmogorov-Smirnov test. \code{"pointwise"} uses a pointwise band (see
#' documentation for argument \code{alpha} for more information). \code{"ell"}
#' is recommended and is the default.
#' @param prob_pts_method (optional) method used to get probability points for
#' use in a QQ plot. The quantile function will be applied to these points to
#' get the expected values.  When this argument is set to \code{"normal"}
#' (recommended for a normal QQ plot) \code{ppoints(n)} will be used,  which is what
#' most other plotting software uses. When this argument is set to \code{"uniform"}
#' (recommended for a uniform QQ plot) \code{ppoints(n, a=0)}, which are the expected
#' values of the order statistics of Uniform(0, 1), will be used.  Finally,
#'  when this argument is set to \code{"median"} (recommended for all other
#'  distributions) \code{qbeta(.5, c(1:n), c(n:1))} will be used. Under the default
#'  setting, \code{"best_available"}, the probability points as recommended above will
#'  be used. Note that \code{"median"} is suitable for all distributions and is
#'  particularly recommended when alpha is large.
#'
#' @return A list with components
#' \itemize{
#'   \item lower_bound - Numeric vector of length \code{n} containing the lower bounds
#'   for the acceptance regions of the test corresponding to each order statistic.
#'   These form the lower boundary of the testing band for the QQ-plot.
#'   \item upper_bound - Numeric vector of length \code{n} containing the upper bounds
#'   for the acceptance regions of the test corresponding to each order statistic.
#'   These form the upper boundary of the testing band for the QQ-plot.
#'   \item expected_value - Numeric vector of length \code{n} containing the
#'   exact or approximate expectation (or median) of each order statistic, depending on how
#'   \code{prob_pts_method} is set.
#'   These are the x-coordinates for both the bounds and the data points
#'   if used in a qq-plot. Note that
#'   if adding a band to an already existing plot, it is essential that the same
#'   x-coordinates be used for the bounds as were used to plot the data. Thus,
#'   if some other x-coordinates have been used to plot the data those same
#'   x-coordinates should always be used instead of this vector to plot the bounds.
#'   \item dparams - List of arguments used to apply \code{distribution} to
#'   \code{obs} (if observations are provided). If the user provides parameters,
#'   then these parameters will simply be returned. If parameters are estimated
#'   from the data, then the estimated parameters will be returned.
#' }
#' @export
#'
#' @examples
#'
#' # Get ell level .05 QQ testing band for normal(0, 1) distribution with 100 observations
#' band <- get_qq_band(n = 100)
#'
#' # Get ell level .05 QQ testing band for normal distribution with unknown parameters
#' obs <- rnorm(100)
#' band <- get_qq_band(obs = obs)
#'
#' # Get ell level .05 QQ testing band for t(2) distribution with 100 observations
#' band <- get_qq_band(
#'   n = 100, distribution = qt, dparams = list(df = 2)
#' )
#'
get_qq_band <- function(
  n,
  obs,
  alpha = .05,
  distribution = qnorm,
  dparams = list(),
  ell_params = list(),
  band_method = c("ell", "ks", "pointwise"),
  prob_pts_method = c("best_available", "normal", "uniform", "median")
) {

  if (missing(obs) && missing(n)) {

    stop("one of obs or n must be supplied")

  }

  if (!("p" %in% names(formals(distribution)))) {

    stop("distribution function must take 'p' as an argument.")

  }

  band_method <- match.arg(band_method)
  prob_pts_method <- match.arg(prob_pts_method)
  dist_name <- as.character(substitute(distribution))

  if(!missing(obs)) {

    n <- length(obs)
    if (length(dparams) == 0) {

      if (dist_name %in% c("qunif", "punif")) {

        dparams['min'] <- 0
        dparams['max'] <- 1

      }
      else {

        cat("no dparams supplied. Estimating parameters from the data...\n")
        MASS_name <- get_mass_name_from_distr(dist_name, "qq")
        dparams <- estimate_params_from_data(MASS_name, obs)

      }

    }

  }

  if (prob_pts_method == "best_available") {

    prob_pts_method <- get_best_available_prob_pts_method(dist_name)

  }

  if (prob_pts_method == "uniform") {

    raw_exp_pts <- ppoints(n, a=0)

  } else if (prob_pts_method == "normal") {

    raw_exp_pts <- ppoints(n)

  } else if (prob_pts_method == "median") {

    raw_exp_pts <- qbeta(.5, c(1:n), c(n:1))

  }

  tryCatch(
    expr = {

      exp_pts <- do.call(distribution, c(list(p=raw_exp_pts), dparams))

    },
    error = function(e){
      message('Error applying distribution to calculated bounds.')
      message('Did you specify all necessary parameters of distribution?')
      message(e)
    }
  )

  if (band_method == "ell") {

    ell_params["n"] <- n
    ell_params["alpha"] <- alpha
    ell_bounds <- do.call(get_bounds_two_sided, ell_params)
    lower_bound <- ell_bounds$lower_bound
    upper_bound <- ell_bounds$upper_bound

  } else if (band_method == "ks") {

    probs <- ppoints(n)
    epsilon <- sqrt((1 / (2 * n)) * log(2 / alpha))
    lower_bound <- pmax(probs - epsilon, rep(0, n))
    upper_bound <- pmin(probs + epsilon, rep(1, n))

  } else if (band_method == "pointwise") {

    conf.int <- 1 - alpha
    conf <- c(alpha / 2, conf.int + alpha / 2)
    lower_bound <- qbeta(conf[1], 1:n, n:1)
    upper_bound <- qbeta(conf[2], 1:n, n:1)

  }

  lower_bound <- do.call(distribution, c(list(p = lower_bound), dparams))
  upper_bound <- do.call(distribution, c(list(p = upper_bound), dparams))

  return(
    list(
      lower_bound = lower_bound,
      upper_bound = upper_bound,
      expected_value = exp_pts,
      dparams = dparams
    )
  )

}

#' Check Validity of One-Sided Bounds
#'
#' Given bounds for a one sided test, this checks that none of
#' the bounds fall outside of [0, 1].
#'
#' @param upper_bounds Numeric vector where the ith component is the upper bound
#' for the ith order statistic.
#'
#' @return None
#'
check_bounds_one_sided <- function(upper_bounds) {

  if(any(upper_bounds > 1) || any(upper_bounds < 0)) {

    stop("Not all bounds between 0 and 1 (inclusive)")

  }

  if(is.unsorted(upper_bounds)) {

    stop("Bounds are not sorted")

  }

  if(any(duplicated(upper_bounds))) {

    stop("Not all values of the bounds are distinct")

  }

}


#' Calculates Global Significance Level From Simultaneous One-Sided Bounds for Rejection Region
#'
#' For a one-sided test of uniformity of i.i.d. observations on the unit interval,
#' this function will determine the significance level as a function of the rejection region.
#' Suppose \eqn{n} observations are drawn i.i.d. from some CDF F(x) on the unit interval,
#' and it is desired to test the null hypothesis that F(x) = x for all x in (0, 1) against
#' the one-sided alternative F(x) > x. Suppose the acceptance region for the test is
#' described by a set of lower bounds, one for each order statistic.
#' Given the lower bounds, this function calculates the significance level of the test where the
#' null hypothesis is rejected if at least one of the order statistics
#' falls below its corresponding lower bound.
#'
#' Uses the method of Moscovich and Nadler (2016) as implemented in Crossprob (Moscovich 2020).
#'
#' @param bounds Numeric vector where the ith component is the lower bound
#' for the ith order statistic. The components must lie in [0, 1], and each component must be
#' greater than or equal to the previous one.
#'
#' @references
#' \itemize{
#' \item{\href{https://www.sciencedirect.com/science/article/abs/pii/S0167715216302802}{
#' Moscovich, Amit, and Boaz Nadler. "Fast calculation of boundary crossing probabilities for Poisson processes."
#' Statistics & Probability Letters 123 (2017): 177-182.}}
#' \item{\href{https://github.com/mosco/crossing-probability}{
#' Amit Moscovich (2020). Fast calculation of p-values for one-sided
#' Kolmogorov-Smirnov type statistics. arXiv:2009.04954}}
#' }
#'
#' @useDynLib qqconf
#'
#' @return Global significance level
#'
#' @examples
#' # For X1, X2, X3 i.i.d. unif(0, 1),
#' # calculate 1 - P(X(1) > .1 and X(2) > .5 and X(3) > .8),
#' # where X(1), X(2), and X(3) are the order statistics.
#' get_level_from_bounds_one_sided(bounds = c(.1, .5, .8))
#'
#' @export
get_level_from_bounds_one_sided <- function(bounds) {

  check_bounds_one_sided(bounds)
  n <- length(bounds)

  if(n == 1) {

    return(bounds[1]) # checks for edge case of one bound

  } else{

    return(get_level_from_bounds_two_sided(bounds, rep(1, n)))

  }

}


#' Calculates Rejection Region of One-Sided Equal Local Levels Test
#'
#' The context is that n i.i.d. observations are assumed to be drawn
#' from some distribution on the unit interval with c.d.f. F(x), and it is
#' desired to test the null hypothesis that F(x) = x for all x in (0,1),
#' referred to as the "global null hypothesis," against the alternative F(x) > x for at least one x in (0, 1).
#' An "equal local levels" test is used, in which each of the n order statistics is
#' tested for significant deviation from its null distribution by a one-sided test
#' with significance level \eqn{\eta}.  The global null hypothesis is rejected if at
#' least one of the order statistic tests is rejected at level eta, where eta is
#' chosen so that the significance level of the global test is alpha.
#' Given the size of the dataset n and the desired global significance level alpha,
#' this function calculates the local level eta and the acceptance/rejection regions for the test.
#' The result is a set of lower bounds, one for each order statistic.
#' If at least one order statistic falls below the corresponding bound,
#' the global test is rejected.
#'
#'
#'
#' @param alpha Desired global significance level of the test.
#' @param n Size of the dataset.
#' @param tol (Optional) Relative tolerance of the \code{alpha} level of the
#' simultaneous test. Defaults to 1e-8.
#' @param max_it (Optional) Maximum number of iterations of Binary Search Algorithm
#' used to find the bounds. Defaults to 100 which should be much larger than necessary
#' for a reasonable tolerance.
#'
#' @return A list with components
#' \itemize{
#'   \item bound - Numeric vector of length \code{n} containing the lower bounds of the acceptance regions for the test of each order statistic.
#'   \item x - Numeric vector of length \code{n} containing the expectation of each order statistic. These are the x-coordinates for the bounds if used in a qq-plot.
#'   The value is \code{c(1:n) / (n + 1)}.
#'   \item local_level - Significance level \eqn{\eta} of the local test on each individual order statistic. It is equal for all order
#'   statistics and will be less than \code{alpha} for all \code{n} > 1.
#' }
#'
#' @examples
#' get_bounds_one_sided(alpha = .05, n = 10, max_it = 50)
#'
#'
#' @export
get_bounds_one_sided <- function(alpha, n, tol = 1e-8, max_it = 100) {

  if (alpha >= 1 || alpha <= 0) {

    stop("alpha must be between 0 and 1 (exclusive).")

  }

  if (as.integer(n) != n) {

    stop("n must be an integer")

  }

  if (n < 1) {

    stop("n must be greater than 0")

  }

  eta_high <- alpha
  eta_low <- alpha / n
  eta_curr <- eta_low + (eta_high - eta_low) / 2
  n_it <- 0

  while (n_it < max_it) {

    n_it <- n_it + 1
    h_vals <- stats::qbeta(eta_curr, 1:n, n:1)

    test_alpha <- get_level_from_bounds_one_sided(h_vals)

    if (abs(test_alpha - alpha) / alpha <= tol) break

    if (test_alpha > alpha) {

      eta_high <- eta_curr
      eta_curr <- eta_curr - (eta_curr - eta_low) / 2

    } else if (test_alpha < alpha) {

      eta_low <- eta_curr
      eta_curr <- eta_curr + (eta_high - eta_curr) / 2

    }

    if(n_it == max_it) {

      warning("Maximum number of iterations reached.")

    }

  }

  alpha_vec <- seq(from = 1, to = n, by = 1)
  beta_vec <- n - alpha_vec + 1
  order_stats_mean <- alpha_vec / (alpha_vec + beta_vec)

  return(list(bound = h_vals, x = order_stats_mean, local_level = eta_curr))

}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

fft_get_level_from_bounds_two_sided <- function(b, B) {
    .Call(`_qqconf_fft_get_level_from_bounds_two_sided`, b, B)
}

---
title: "qqconf: An R Package for Putting Testing Bands on QQ and PP Plots"
author: 
  - Eric Weine^[University of Chicago, ericweine15@gmail.com]
  - Mary Sara McPeek^[University of Chicago, mcpeek@uchicago.edu]
  - Mark Abney^[University of Chicago, abney@uchicago.edu]
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_vignette:
    toc: true
    toc_depth: 4
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

The `qqconf` package extends `base` graphics capabilities to put simultaneous testing bands on Quantile-Quantile (QQ) and Probability-Probability (PP) plots. We support plots for any distribution with a Cumulative Distribution Function (CDF) implemented in R. For the creation of simultaneous testing bands, we support the Kolmogorov-Smirnov (Kolmogorov 1941; Smirnov 1944) method as well as what we refer to as the Equal Local Levels (ELL) method, which conducts an $\eta$ level test on each order statistic of the sample supplied such that the global testing bands have some desired $\alpha$ Type I error rate. For more information on the computation and properties of ELL, please see our section about ELL Testing Bounds Generation and our [paper](https://arxiv.org/abs/2111.15082). 

## Installation

Our package is available on CRAN and can be installed by running:

``` {r eval = F} 
install.packages("qqconf")
```

## Usage

### Plotting

The package contains two functions for creating plots, `qq_conf_plot` and `pp_conf_plot` to create QQ plots and PP plots, respectively. These two functions have identical interfaces, with the exception that `qq_conf_plot` requires the input of a quantile function (e.g. `qnorm`) and `pp_conf_plot` requires the input of a distribution function (e.g. `pnorm`). Thus, any information below about parameters that can be fed to the `qq_conf_plot` function also apply to `pp_conf_plot`, and vice versa.

#### Basics / Parameter Estimation

First, we load in `qqconf`

```{r message = F}
require(qqconf)
```

Then, we generate data from a standard normal distribution:

```{r}
set.seed(0)
sample <- rnorm(n = 100, mean = 0, sd = 1)
```

Now, if we did not know the distribution that our sample came from but instead wanted to test if it was sampled i.i.d from a $N(0, 1)$ distribution, we could create a QQ plot to compare the quantiles of the sample to the theoretical quantiles of a $N(0, 1)$ distribution.

To do this, we call `qq_conf_plot` as follows:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1)
)
```

By default, this function creates a QQ plot with ELL testing bounds calculated with a Type I error rate of .05. Note here that `dparams` is not a required parameter. If we were to call `qq_conf_plot` without specifying `dparams`, then our code would estimate the mean and variance of the sample assuming it comes from a normal distribution as is specified by the `distribution` parameter. More generally, for any continuous distribution that is neither normal nor uniform, we use maximum liklihood estimation to estimate all parameters of the distribution. For the uniform distribution, we do not support parameter estimation but instead default to $U(0, 1)$ and allow the user to specify a custom `max` and `min`. For the normal distribution, because it is so commonly used as the reference distribution in QQ plots, we experimented with different methods of parameter estimation and found that estimating the mean of the distribution as the median of the sample and the standard deviation as $S_{n}$ as proposed by Rousseeuw and Croux (Rousseeuw, Crow 1993) had well calibrated Type I error rates. When we used the MLEs for the normal distribution, we found that the testing bounds produced were conservative. Note that the testing bounds produced for other distributions with parameters estimated via MLE will also likely be conservative. For more information on parameter estimation, please see section 2.6 of our [paper](https://arxiv.org/abs/2111.15082).

#### Modifying Visual Features

##### General Plot Features

Parameter estimation aside, we have now created a QQ plot that compares our sample to $N(0, 1)$. While this plot is informative, there are a number of potential modifications we may want to make to this plot for ease of viewing. First, because the top and the bottom of the confidence bands are cut off in this plot, we may want to expand the y-axis limits. We do this by simply adding a `ylim` argument to the function call:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4)
)
```

In fact, because `qq_conf_plot` takes `...` as an argument, any other parameter normally passed to the `plot` function in `base` can be passed to `qq_conf_plot`. If, for instance, we wanted to modify the axis labels, we could write:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  xlab = "More Informative Title"
)
```

However, while the `...` argument can be used for more general features of the plot like axis titles, because there are so many separate objects in the plot (points, lines, bands), we've added additional arguments so that the user can easily specify the visual features of any element of the plot.

##### Points

If we want to modify the appearance of the points on the plot, we can use the optional `points_params` argument. This argument is a list that takes any arguments that can be passed to the `points` function in `base`. For instance, if we wanted to make the points smaller, we could do the following.

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  points_params = list(cex = .5) # makes points smaller
)
```

For more options, see the documentation of the `points` function.

##### Expectation Line

If we want to modify the line in the plot that indicates a perfect fit between the reference distribution and the sample, we can use the `line_params` argument. This list can take any parameters that can be passed to the `lines` function and will modify the plot accordingly. If, for instance, we wanted to make the line red, we would do the following:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  line_params = list(col="red") # makes expectation line red
)
```

Note, that if we don't want this line to show up on the plot at all, we can pass in `type = "n"` to `line_params`.

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  line_params = list(type="n") # removes expectation line
)
```

Again, for more information see the documentation for the `lines` function.

##### Testing Bands

By default, `qq_conf_plot` does not plot pointwise testing bands on the plot. Pointwise testing bands, in contrast to simultaneous testing bands, are anti-conservative, as they ignore the multiple testing problem inherent in QQ plots and simply conduct a test on each order statistic. To add these pointwise bands, one can simply set the `plot_pointwise` parameter to `True` in the `qq_conf_plot` function. To modify their appearance, one can use the `pointwise_lines_params` parameter. Because this parameter list is simply passed to the `lines` function, it works exactly the same way as the `line_params` parameter.

Finally, if we want to modify the appearance of the simultaneous testing bands, we can use the `polygon_params` parameter. As the name suggests, arguments in this parameter list are passed to the `polygon` function in `base` and modify the testing bands accordingly. Note that by default, `border` is set to `NA` and `col` is set to `grey`. If the user overrides the default option, all defaults of `polygon` are used unless otherwise specified, which does not set `border` to `NA` and `col` to `grey`. If, for instance, we wanted to change the color of the shading, we would do the following:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
qq_conf_plot(
  obs = sample, 
  distribution = qnorm,
  dparams = list(mean = 0, sd = 1),
  ylim = c(-4, 4),
  polygon_params = list(col = 'powderblue', border = NA) # change shading and keep no border
)
```

#### Scaling and Data Transformations

In addition to general visual parameters to modify the elements of the plot, we've also included a few parameters that we found to be useful in particular cases when making a QQ or PP plot. Especially when $n$ is large, it can be difficult to see the relevant parts of the plot.

To demonstrate this, we will generate 10,000 points from a $Beta(1, 1.05)$ distribution and test this sample against a $U(0, 1)$ distribution. 

```{r}
sample <- rbeta(n = 10000, shape1 = 1.0, shape2 = 1.05)
```

Because a QQ plot and PP plot are equivalent for the uniform distribution, we will switch to PP plots to demonstrate the interface.

##### Differencing 

Normally in a PP plot, the expected probabilities are plotted on the x-axis and the theoretical probabilities are plotted on the y-axis. 

```{r fig.align = "center", fig.width = 7, fig.height = 5}
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1)
)
```

However, in this plot it is very difficult to see anything because the individual points are so small and the magnitude of the deviation from the reference distribution is not very large.

Instead, if we plot the expected probabilities on the x-axis and the observed probabilities - the expected probabilities on the y-axis by setting the `difference` parameter to `TRUE`, things become much easier to see:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  difference = TRUE, # Make y-axis differenced
  ylim = c(-.0225, .0225)
)
```

##### log Scaling

Sometimes, we are only interested in deviations of the sample from one tail of the reference distribution. For instance, if we have a sample of $n$ p-values that we would like to compare to the $U(0, 1)$ distribution for the sake of global null hypothesis testing, we're generally only interested in p-values that are small. In this case, especially with large $n$, it can be helpful to transform the probability values onto the $-log10$ scale.

To demonstrate this, we will generate 10,000 points from a mixture of a $Beta(.25, 1)$ and a $U(0, 1)$ and test this sample against a $U(0, 1)$ reference distribution. 

```{r}
mix <- distr::UnivarMixingDistribution(
  distr::Beta(shape1 = .25, shape2 = 1), 
  distr::Unif(),
  mixCoeff=c(.01, .99)
)

sampler <- distr::r(mix)

sample <- sampler(10000)
```

Again, if we make a standard PP plot the relevant parts of the plot are very difficult to make out.

```{r fig.align = "center", fig.width = 7, fig.height = 5}
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1)
)
```

Even if we set `difference` to `TRUE`, it's still difficult to see the left tail of the distribution.

```{r fig.align = "center", fig.width = 7, fig.height = 5}
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  difference = TRUE
)
```

However, setting `log10` to `TRUE` makes the plot much easier to see.

```{r fig.align = "center", fig.width = 7, fig.height = 5}
pp_conf_plot(
  obs = sample, 
  distribution = punif,
  points_params = list(cex=.1),
  log10 = TRUE
)
```

Note, that if we want to highlight the right tail of the distribution as opposed to the left tail, we can set `log10` to `TRUE` and the `right_tail` parameter to `TRUE`. This results in a plot with the data transformed onto the `log10` scale instead of the `-log10` scale. 

### ELL Testing Bounds Generations

In addition to the plotting interfaces explained above, `qqconf` provides functions to directly produce testing bounds generated by the equal local levels method. Functions exist for both two-sided and one-sided testing. For information on the generation of testing bounds using equal local levels, please see section 2 of our [paper](https://arxiv.org/abs/2111.15082).

#### Two-Sided Testing

Given a desired global Type I error rate $\alpha$ and a sample size $n$, we can generate two-sided bounds using the `get_bounds_two_sided` function. Below, we generate such bounds for $n = 100$ and $\alpha = .05$:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
bounds <- get_bounds_two_sided(alpha = .05, n = 100)
```

In this case, the `bounds` object is a list with a number of objects (for more information see the documentation), but the lower bounds can be extracted with `bounds$lower_bounds` and the upper bounds with `bounds$upper_bounds`.

#### One-Sided Testing

In addition to two-sided testing, `qqconf` also provides functionality for one-sided testing. 

Given a desired global Type I error rate $\alpha$ and a sample size $n$, we can generate such bounds using the `get_bounds_one_sided` function. Below, we generate such bounds for $n = 100$ and $\alpha = .05$:

```{r fig.align = "center", fig.width = 7, fig.height = 5}
bounds <- get_bounds_one_sided(alpha = .05, n = 100)
```

From the `bounds` object, the upper bounds can be extracted via `bounds$bound`. Again, this function returns a list and more information can be extracted from `bounds`.

## Session info

All plots above were generated with the following session information:

```{r session-info}
sessionInfo()
```

## References

* [Rousseeuw, Peter J., and Christophe Croux. "Alternatives to the median absolute deviation." Journal of the American Statistical association 88.424 (1993): 1273-1283.]

* [Kolmogorov A (1941). “Confidence limits for an unknown distribution function.” The annals of mathematical statistics, 12(4), 461–463.]

* [Smirnov NV (1944). “Approximate laws of distribution of random variables from em- pirical data.” Uspekhi Matematicheskikh Nauk, (10), 179–206.]

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{get_bounds_two_sided}
\alias{get_bounds_two_sided}
\title{Calculates Rejection Region of Two-Sided Equal Local Levels Test.}
\usage{
get_bounds_two_sided(
  alpha,
  n,
  tol = 1e-08,
  max_it = 100,
  method = c("best_available", "approximate", "search")
)
}
\arguments{
\item{alpha}{Desired global significance level of the test.}

\item{n}{Size of the dataset.}

\item{tol}{(optional) Relative tolerance of the \code{alpha} level of the
simultaneous test. Defaults to 1e-8. Used only if \code{method} is set to
"search" or if method is set to "best_available" and the best available
method is a search.}

\item{max_it}{(optional) Maximum number of iterations of Binary Search Algorithm
used to find the bounds. Defaults to 100 which should be much larger than necessary
for a reasonable tolerance. Used only if \code{method} is set to
"search" or if method is set to "best_available" and the best available
method is a search.}

\item{method}{(optional) Argument indicating if the calculation should be done using a highly
accurate approximation, "approximate", or if the calculations should be done using an exact
binary search calculation, "search". The default is "best_available" (recommended), which uses the exact search
when either (i) the approximation isn't available or (ii) the approximation is available but isn't highly accurate and the search method
isn't prohibitively slow (occurs for small to moderate \code{n} with \code{alpha} = .1).
Of note, the approximate method is only available for alpha values of .1, .05, and .01. In the case of alpha = .05 or .01, the
approximation is highly accurate for all values of \code{n} up to at least \code{10^6}.}
}
\value{
A list with components
\itemize{
  \item lower_bound - Numeric vector of length \code{n} containing the lower bounds
  for the acceptance regions of the test of each order statistic.
  \item upper_bound - Numeric vector of length \code{n} containing the upper bounds
  for the acceptance regions of the test of each order statistic.
  \item x - Numeric vector of length \code{n} containing the expectation of each order statistic.
  These are the x-coordinates for the bounds if used in a pp-plot.
  The value is \code{c(1:n) / (n + 1)}.
  \item local_level - Significance level \eqn{\eta} of the local test on each individual order statistic. It is equal for all order
  statistics and will be less than \code{alpha} for all \code{n} > 1.
}
}
\description{
The context is that n i.i.d. observations are assumed to be drawn
from some distribution on the unit interval with c.d.f. F(x), and it is
desired to test the null hypothesis that F(x) = x for all x in (0,1),
referred to as the "global null hypothesis," against a two-sided alternative.
An "equal local levels" test is used, in which each of the n order statistics is
tested for significant deviation from its null distribution by a 2-sided test
with significance level \eqn{\eta}.  The global null hypothesis is rejected if at
least one of the order statistic tests is rejected at level \eqn{\eta}, where \eqn{\eta} is
chosen so that the significance level of the global test is alpha.
Given the size of the dataset n and the desired global significance level alpha,
this function calculates the local level \eqn{\eta} and the acceptance/rejection regions for the test.
There are a set of n intervals, one for each order statistic.
If at least one order statistic falls outside the corresponding interval,
the global test is rejected.
}
\examples{
get_bounds_two_sided(alpha = .05, n = 100)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_qq_distribution_from_pp_distribution}
\alias{get_qq_distribution_from_pp_distribution}
\title{Get Quantile Distribution from Probability Distribution}
\usage{
get_qq_distribution_from_pp_distribution(dname)
}
\arguments{
\item{dname}{probability distribution (e.g. \code{pnorm})}
}
\value{
quantile distribution (e.g. \code{qnorm}).
}
\description{
Get Quantile Distribution from Probability Distribution
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ppplot.R
\name{pp_conf_plot}
\alias{pp_conf_plot}
\title{PP Plot with Simultaneous and Pointwise Testing Bounds.}
\usage{
pp_conf_plot(
  obs,
  distribution = pnorm,
  method = c("ell", "ks"),
  alpha = 0.05,
  difference = FALSE,
  log10 = FALSE,
  right_tail = FALSE,
  add = FALSE,
  dparams = list(),
  bounds_params = list(),
  line_params = list(),
  plot_pointwise = FALSE,
  pointwise_lines_params = list(),
  points_params = list(),
  polygon_params = list(border = NA, col = "gray"),
  prob_pts_method = c("uniform", "median", "normal"),
  ...
)
}
\arguments{
\item{obs}{The observed data.}

\item{distribution}{The probability function for the specified distribution. Defaults to \code{pnorm}.
Custom distributions are allowed as long as all parameters are supplied in dparams.}

\item{method}{Method for simultaneous testing bands. Must be either "ell" (equal local levels test), which applies a level \eqn{\eta} pointwise
test to each order statistic such that the Type I error of the global test is \code{alpha}, or "ks" to apply a
Kolmogorov-Smirnov test. "ell" is recommended.}

\item{alpha}{Type I error of global test of whether the data come from the reference distribution.}

\item{difference}{Whether to plot the difference between the observed and
expected values on the vertical axis.}

\item{log10}{Whether to plot axes on -log10 scale (e.g. to see small p-values).}

\item{right_tail}{This argument is only used if \code{log10} is \code{TRUE}. When \code{TRUE},
the x-axis is -log10(1 - Expected Probability) and the y-axis is -log10(1 - Observed Probability).
When \code{FALSE} (default) the x-axis is -log10(Expected Probability) and the y-axis is
-log10(Observed Probability). The argument should be set to \code{TRUE} to make
observations in the right tail of the distribution easier to see, and set to false to make the
observations in the left tail of the distribution easier to see.}

\item{add}{Whether to add points to an existing plot.}

\item{dparams}{List of additional arguments for the probability function of the distribution
(e.g. df=1). Note that if any parameters of the distribution are specified, parameter estimation will not be performed
on the unspecified parameters, and instead they will take on the default values set by the distribution function.
For the uniform distribution, parameter estimation is not performed, and
the default parameters are max = 1 and min = 0.
For other distributions parameters will be estimated if not provided.
For the normal distribution, we estimate the mean as the median and the standard deviation as \eqn{Sn} from the paper by Rousseeuw and Croux 1993
"Alternatives to the Median Absolute Deviation". For all other distributions besides uniform and normal,
the code uses MLE to estimate the parameters. Note that estimation is not implemented for custom distributions, so all
parameters of the distribution must be provided by the user.}

\item{bounds_params}{List of optional arguments for \code{get_bounds_two_sided}.
(i.e. \code{tol}, \code{max_it}, \code{method}).}

\item{line_params}{arguments passed to the line function to modify the line that indicates a perfect fit of the
reference distribution.}

\item{plot_pointwise}{Boolean indicating whether pointwise bounds should be added to the plot}

\item{pointwise_lines_params}{arguments passed to the \code{lines} function that modifies pointwise bounds when plot_pointwise is
set to TRUE.}

\item{points_params}{arguments to be passed to the \code{points} function to plot the data.}

\item{polygon_params}{Arguments to be passed to the polygon function to construct simultaneous confidence region.
By default \code{border} is set to NA and \code{col} is set to grey.}

\item{prob_pts_method}{(optional) method used to get probability points for
plotting. The default value, \code{"uniform"}, results in
\code{ppoints(n, a=0)}, which are the expected
values of the order statistics of Uniform(0, 1).  When
 this argument is set to \code{"median"}, \code{qbeta(.5, c(1:n), c(n:1))},
 the medians of the order statistics of Uniform(0, 1) will be used. For a
 PP plot, there is no particular theoretical justification for setting this
 argument to \code{"normal"}, which results in \code{ppoints(n)}, but it is
 an option because it is used in some
 other packages. When \code{alpha} is large, \code{"median"} is recommended.}

\item{...}{Additional arguments passed to the plot function.}
}
\value{
None, PP plot is produced.
}
\description{
Create a pp-plot with with a shaded simultaneous acceptance region and,
optionally, lines for a point-wise region. The observed values are plotted
against their expected values had they come from the specified distribution.
}
\details{
If any of the points of the pp-plot fall outside the simultaneous acceptance region for the selected
level alpha test, that means that we can reject the null hypothesis that the data are i.i.d. draws from the
specified distribution. If \code{difference} is set to TRUE, the vertical axis plots the
observed probability minus expected probability. If pointwise bounds are used, then on average, alpha * n of the points will fall outside
the bounds under the null hypothesis, so the chance that the pp-plot has any points falling outside of the pointwise bounds
is typically much higher than alpha under the null hypothesis. For this reason, a simultaneous region is preferred.
}
\examples{
set.seed(0)
smp <- rnorm(100)

# Plot PP plot against normal distribution with mean and variance estimated
pp_conf_plot(
  obs=smp
)

# Make same plot on -log10 scale to highlight the left tail,
# with radius of plot circles also reduced by .5
pp_conf_plot(
  obs=smp,
  log10 = TRUE,
  points_params = list(cex = .5)
)

# Make same plot with difference between observed and expected values on the y-axis
pp_conf_plot(
  obs=smp,
  difference = TRUE
)

# Make same plot with samples plotted as a blue line, expected value line plotted as a red line,
# and pointwise bounds plotted as black lines
pp_conf_plot(
  obs=smp,
  plot_pointwise = TRUE,
  points_params = list(col="blue", type="l"),
  line_params = list(col="red")
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{estimate_params_from_data}
\alias{estimate_params_from_data}
\title{Estimate Parameters from Data}
\usage{
estimate_params_from_data(distr_name, obs)
}
\arguments{
\item{distr_name}{\code{MASS} name of distribution}

\item{obs}{observation vector}
}
\value{
list of distribution parameters
}
\description{
For select distributions, parameters are estimated from data. Generally,
the MLEs are used. However, for the normal distribution we use robust
estimators.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{get_asymptotic_approx_corrected_alpha}
\alias{get_asymptotic_approx_corrected_alpha}
\title{Calculates Approximate Local Level}
\usage{
get_asymptotic_approx_corrected_alpha(n, alpha)
}
\arguments{
\item{n}{Number of tests to do}

\item{alpha}{Global type I error rate \eqn{\alpha} of the tests}
}
\value{
Approximate local level
}
\description{
This function uses the approximation from Gontscharuk & Finner's Asymptotics of
goodness-of-fit tests based on minimum p-value statistics (2017) to approximate
local levels for finite sample size. We use these authors constants for \eqn{\alpha} = .1, and .05,
and for \eqn{\alpha} = .01 we use a slightly different approximation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_extended_quantile}
\alias{get_extended_quantile}
\title{Get Quantile for First and Last Point of QQ or PP Plot}
\usage{
get_extended_quantile(exp_pts_method, n)
}
\arguments{
\item{exp_pts_method}{method used to derive expected points}

\item{n}{sample size}
}
\value{
list with low and high point
}
\description{
Get Quantile for First and Last Point of QQ or PP Plot
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one_sided.R
\name{get_bounds_one_sided}
\alias{get_bounds_one_sided}
\title{Calculates Rejection Region of One-Sided Equal Local Levels Test}
\usage{
get_bounds_one_sided(alpha, n, tol = 1e-08, max_it = 100)
}
\arguments{
\item{alpha}{Desired global significance level of the test.}

\item{n}{Size of the dataset.}

\item{tol}{(Optional) Relative tolerance of the \code{alpha} level of the
simultaneous test. Defaults to 1e-8.}

\item{max_it}{(Optional) Maximum number of iterations of Binary Search Algorithm
used to find the bounds. Defaults to 100 which should be much larger than necessary
for a reasonable tolerance.}
}
\value{
A list with components
\itemize{
  \item bound - Numeric vector of length \code{n} containing the lower bounds of the acceptance regions for the test of each order statistic.
  \item x - Numeric vector of length \code{n} containing the expectation of each order statistic. These are the x-coordinates for the bounds if used in a qq-plot.
  The value is \code{c(1:n) / (n + 1)}.
  \item local_level - Significance level \eqn{\eta} of the local test on each individual order statistic. It is equal for all order
  statistics and will be less than \code{alpha} for all \code{n} > 1.
}
}
\description{
The context is that n i.i.d. observations are assumed to be drawn
from some distribution on the unit interval with c.d.f. F(x), and it is
desired to test the null hypothesis that F(x) = x for all x in (0,1),
referred to as the "global null hypothesis," against the alternative F(x) > x for at least one x in (0, 1).
An "equal local levels" test is used, in which each of the n order statistics is
tested for significant deviation from its null distribution by a one-sided test
with significance level \eqn{\eta}.  The global null hypothesis is rejected if at
least one of the order statistic tests is rejected at level eta, where eta is
chosen so that the significance level of the global test is alpha.
Given the size of the dataset n and the desired global significance level alpha,
this function calculates the local level eta and the acceptance/rejection regions for the test.
The result is a set of lower bounds, one for each order statistic.
If at least one order statistic falls below the corresponding bound,
the global test is rejected.
}
\examples{
get_bounds_one_sided(alpha = .05, n = 10, max_it = 50)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{get_qq_band}
\alias{get_qq_band}
\title{Create QQ Plot Testing Band}
\usage{
get_qq_band(
  n,
  obs,
  alpha = 0.05,
  distribution = qnorm,
  dparams = list(),
  ell_params = list(),
  band_method = c("ell", "ks", "pointwise"),
  prob_pts_method = c("best_available", "normal", "uniform", "median")
)
}
\arguments{
\item{n, obs}{either a number of observations (specified by setting \code{n}),
or a numeric vector of observations (specified by setting \code{obs}). One
argument must be specified. If all parameters of \code{distribution}
are known, then the testing band only depends on the number of observations \code{n}.
Thus, providing \code{n} is simpler when all parameters
of \code{distribution} are known and specified via \code{dparams}
(or when using all default parameter choices of \code{distribution} is desired).
If estimating parameters from the data is preferred, \code{obs} should be
specified and estimation will take place as described in the documentation for
argument \code{dparams}.}

\item{alpha}{(optional) desired significance level of the testing band. If \code{method}
is set to \code{"ell"} or \code{"ks"}, then this is the global significance
level of the testing band. If \code{method} is set to \code{"pointwise"}, then
the band is equivalent to simply conducting a level \code{alpha} test on each
order statistic individually. Pointwise bands will generally have much
larger global Type I error than \code{alpha}. Defaults to \code{.05}.}

\item{distribution}{The quantile function for the specified distribution.
Defaults to \code{qnorm}, which is appropriate for testing normality
of the observations in a QQ plot.}

\item{dparams}{(optional)  List of additional arguments for the \code{distribution} function
(e.g. df=1). If \code{obs} is not specified and this argument is left blank,
the default arguments of \code{distribution} are used. If \code{obs} is specified and this argument is left blank,
parameters are estimated from the data (except if \code{distribution} is set to \code{qunif},
in which case no estimation occurs and the default parameters are \code{max = 1} and \code{min = 0}).
For the normal distribution, we estimate the mean as the median and the standard deviation as \eqn{Sn} from the paper by Rousseeuw and Croux 1993
"Alternatives to the Median Absolute Deviation". For all other distributions besides uniform and normal,
the code uses MLE to estimate the parameters. Note that if any parameters of the distribution are specified
in \code{dparams}, parameter estimation will not be performed
on the unspecified parameters, and instead they will take on the default values
set by \code{distribution}.}

\item{ell_params}{(optional) list of optional arguments for \code{get_bounds_two_sided}
(i.e. \code{tol}, \code{max_it}, \code{method}). Only used if \code{method}
is set to \code{"ell"}}

\item{band_method}{(optional) method for creating the testing band. The default,
\code{"ell"} uses the equal local levels method
(see \code{get_bounds_two_sided} for more information). \code{"ks"} uses
the Kolmogorov-Smirnov test. \code{"pointwise"} uses a pointwise band (see
documentation for argument \code{alpha} for more information). \code{"ell"}
is recommended and is the default.}

\item{prob_pts_method}{(optional) method used to get probability points for
use in a QQ plot. The quantile function will be applied to these points to
get the expected values.  When this argument is set to \code{"normal"}
(recommended for a normal QQ plot) \code{ppoints(n)} will be used,  which is what
most other plotting software uses. When this argument is set to \code{"uniform"}
(recommended for a uniform QQ plot) \code{ppoints(n, a=0)}, which are the expected
values of the order statistics of Uniform(0, 1), will be used.  Finally,
 when this argument is set to \code{"median"} (recommended for all other
 distributions) \code{qbeta(.5, c(1:n), c(n:1))} will be used. Under the default
 setting, \code{"best_available"}, the probability points as recommended above will
 be used. Note that \code{"median"} is suitable for all distributions and is
 particularly recommended when alpha is large.}
}
\value{
A list with components
\itemize{
  \item lower_bound - Numeric vector of length \code{n} containing the lower bounds
  for the acceptance regions of the test corresponding to each order statistic.
  These form the lower boundary of the testing band for the QQ-plot.
  \item upper_bound - Numeric vector of length \code{n} containing the upper bounds
  for the acceptance regions of the test corresponding to each order statistic.
  These form the upper boundary of the testing band for the QQ-plot.
  \item expected_value - Numeric vector of length \code{n} containing the
  exact or approximate expectation (or median) of each order statistic, depending on how
  \code{prob_pts_method} is set.
  These are the x-coordinates for both the bounds and the data points
  if used in a qq-plot. Note that
  if adding a band to an already existing plot, it is essential that the same
  x-coordinates be used for the bounds as were used to plot the data. Thus,
  if some other x-coordinates have been used to plot the data those same
  x-coordinates should always be used instead of this vector to plot the bounds.
  \item dparams - List of arguments used to apply \code{distribution} to
  \code{obs} (if observations are provided). If the user provides parameters,
  then these parameters will simply be returned. If parameters are estimated
  from the data, then the estimated parameters will be returned.
}
}
\description{
Flexible interface for creating a testing band for a Quantile-Quantile (QQ)
plot.
}
\examples{

# Get ell level .05 QQ testing band for normal(0, 1) distribution with 100 observations
band <- get_qq_band(n = 100)

# Get ell level .05 QQ testing band for normal distribution with unknown parameters
obs <- rnorm(100)
band <- get_qq_band(obs = obs)

# Get ell level .05 QQ testing band for t(2) distribution with 100 observations
band <- get_qq_band(
  n = 100, distribution = qt, dparams = list(df = 2)
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{monte_carlo_two_sided}
\alias{monte_carlo_two_sided}
\title{Monte Carlo Simulation for Two-Sided Test}
\usage{
monte_carlo_two_sided(lower_bounds, upper_bounds, num_sims = 1e+06)
}
\arguments{
\item{lower_bounds}{Numeric vector where the ith component is the lower bound
for the ith order statistic. The components must be distinct values in (0, 1) that
are in ascending order.}

\item{upper_bounds}{Numeric vector where the ith component is the lower bound
for the ith order statistic. The values must be in ascending order and
the ith component must be larger than the ith component of the lower bounds.}

\item{num_sims}{(optional) Number of simulations to be run, 1 Million by default.}
}
\value{
Type I Error Rate \eqn{\alpha}
}
\description{
Given bounds for a two sided test on uniform order statistics, this computes
the Type I Error Rate \eqn{\alpha} using simulations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{get_level_from_bounds_two_sided}
\alias{get_level_from_bounds_two_sided}
\title{Calculates Global Significance Level From Simultaneous Two-Sided Bounds for Rejection Region}
\usage{
get_level_from_bounds_two_sided(lower_bounds, upper_bounds)
}
\arguments{
\item{lower_bounds}{Numeric vector where the ith component is the lower bound for the acceptance interval
for the ith order statistic. The components must lie in [0, 1], and each component must be greater than
or equal to the previous one.}

\item{upper_bounds}{Numeric vector of the same length as \code{lower_bounds} where the ith component is the upper bound
for the acceptance interval for the ith order statistic. The components must lie in [0, 1], and each component must be
greater than or equal to the previous one. In addition,
the ith component of \code{upper_bounds} must be greater than or equal to the ith component of \code{lower_bounds}.}
}
\value{
Global Significance Level \eqn{\alpha}
}
\description{
For a test of uniformity of i.i.d. observations on the unit interval, this function will determine the significance
level as a function of the rejection region. Suppose \eqn{n} observations are drawn i.i.d. from some CDF F(x) on the unit interval,
and it is desired to test the null hypothesis that F(x) = x for all x in (0, 1) against a two-sided alternative.
Suppose the acceptance region for the test is described by a set of intervals, one for each order statistic.
Given the bounds for these intervals, this function calculates the significance level of the test where the
null hypothesis is rejected if at least one of the order statistics is outside its corresponding interval.
}
\details{
Uses the method of Moscovich and Nadler (2016) as implemented in Crossprob (Moscovich 2020).
}
\examples{
# For X1, X2 iid unif(0,1), calculate 1 - P(.1 < min(X1, X2) < .6 and .5 < max(X1, X2) < .9)
get_level_from_bounds_two_sided(lower_bounds = c(.1, .5), upper_bounds = c(.6, .9))

# Finds the global significance level corresponding to the local level eta.
# Suppose we reject the null hypothesis that X1, ..., Xn are iid unif(0, 1) if and only if at least
# one of the order statistics X(i) is significantly different from
# its null distribution based on a level-eta
# two-sided test, i.e. we reject if and only if X(i) is outside the interval
# (qbeta(eta/2, i, n - i + 1), qbeta(1 - eta/2, i, n - i + 1)) for at least one i.
# The lines of code below calculate the global significance level of
# the test (which is necessarily larger than eta if n > 1).
n <- 100
eta <- .05
lb <- qbeta(eta / 2, c(1:n), c(n:1))
ub <- qbeta(1 - eta / 2, c(1:n), c(n:1))
get_level_from_bounds_two_sided(lower_bounds = lb, upper_bounds = ub)

}
\references{
\itemize{
\item{\href{https://www.sciencedirect.com/science/article/abs/pii/S0167715216302802}{
Moscovich, Amit, and Boaz Nadler. "Fast calculation of boundary crossing probabilities for Poisson processes."
Statistics & Probability Letters 123 (2017): 177-182.}}
\item{\href{https://github.com/mosco/crossing-probability}{
Amit Moscovich (2020). Fast calculation of p-values for one-sided
Kolmogorov-Smirnov type statistics. arXiv:2009.04954}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_best_available_prob_pts_method}
\alias{get_best_available_prob_pts_method}
\title{Get Best Available Method for Probability Points}
\usage{
get_best_available_prob_pts_method(dist_name)
}
\arguments{
\item{dist_name}{character name of distribution}
}
\value{
character name of best expected points method
}
\description{
Determines name of best method for obtaining expected points for a qq or pp
plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one_sided.R
\name{get_level_from_bounds_one_sided}
\alias{get_level_from_bounds_one_sided}
\title{Calculates Global Significance Level From Simultaneous One-Sided Bounds for Rejection Region}
\usage{
get_level_from_bounds_one_sided(bounds)
}
\arguments{
\item{bounds}{Numeric vector where the ith component is the lower bound
for the ith order statistic. The components must lie in [0, 1], and each component must be
greater than or equal to the previous one.}
}
\value{
Global significance level
}
\description{
For a one-sided test of uniformity of i.i.d. observations on the unit interval,
this function will determine the significance level as a function of the rejection region.
Suppose \eqn{n} observations are drawn i.i.d. from some CDF F(x) on the unit interval,
and it is desired to test the null hypothesis that F(x) = x for all x in (0, 1) against
the one-sided alternative F(x) > x. Suppose the acceptance region for the test is
described by a set of lower bounds, one for each order statistic.
Given the lower bounds, this function calculates the significance level of the test where the
null hypothesis is rejected if at least one of the order statistics
falls below its corresponding lower bound.
}
\details{
Uses the method of Moscovich and Nadler (2016) as implemented in Crossprob (Moscovich 2020).
}
\examples{
# For X1, X2, X3 i.i.d. unif(0, 1),
# calculate 1 - P(X(1) > .1 and X(2) > .5 and X(3) > .8),
# where X(1), X(2), and X(3) are the order statistics.
get_level_from_bounds_one_sided(bounds = c(.1, .5, .8))

}
\references{
\itemize{
\item{\href{https://www.sciencedirect.com/science/article/abs/pii/S0167715216302802}{
Moscovich, Amit, and Boaz Nadler. "Fast calculation of boundary crossing probabilities for Poisson processes."
Statistics & Probability Letters 123 (2017): 177-182.}}
\item{\href{https://github.com/mosco/crossing-probability}{
Amit Moscovich (2020). Fast calculation of p-values for one-sided
Kolmogorov-Smirnov type statistics. arXiv:2009.04954}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{between}
\alias{between}
\title{Shorthand for two numerical comparisons}
\usage{
between(x, gte, lte)
}
\arguments{
\item{x}{numeric value}

\item{gte}{lower bound}

\item{lte}{upper bound}
}
\value{
boolean
}
\description{
Shorthand for two numerical comparisons
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/one_sided.R
\name{check_bounds_one_sided}
\alias{check_bounds_one_sided}
\title{Check Validity of One-Sided Bounds}
\usage{
check_bounds_one_sided(upper_bounds)
}
\arguments{
\item{upper_bounds}{Numeric vector where the ith component is the upper bound
for the ith order statistic.}
}
\value{
None
}
\description{
Given bounds for a one sided test, this checks that none of
the bounds fall outside of [0, 1].
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_mass_name_from_distr}
\alias{get_mass_name_from_distr}
\title{Convert R Distribution Function to MASS Distribution Name}
\usage{
get_mass_name_from_distr(distr, band_type)
}
\arguments{
\item{distr}{R distribution function (e.g. qnorm or pnorm)}

\item{band_type}{one of "qq" (for quantile functions) or "pp" (
for probability functions).}
}
\value{
string of MASS distribution name
}
\description{
Convert R Distribution Function to MASS Distribution Name
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/two_sided.R
\name{check_bounds_two_sided}
\alias{check_bounds_two_sided}
\title{Check Validity of Two-Sided Bounds}
\usage{
check_bounds_two_sided(lower_bounds, upper_bounds)
}
\arguments{
\item{lower_bounds}{Numeric vector where the ith component is the lower bound
for the ith order statistic.}

\item{upper_bounds}{Numeric vector where the ith component is the lower bound
for the ith order statistic.}
}
\value{
None
}
\description{
Given bounds for a two sided test, this checks that none of
the bounds fall outside of [0, 1] and that all upper bounds
are greater than the corresponding lower bounds.
This also ensures the the length of the bounds are the same.
This not meant to be called by the user.
}
